@@ -49,14 +49,14 @@
 
 #include <glib.h>
 
-#if defined(GETTEXT_PACKAGE)
-#include <glib/gi18n-lib.h>
-#else
-#include <glib/gi18n.h>
-#endif
-
 #include <libxfce4util/libxfce4util.h>
 
+#ifdef ENABLE_NLS
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+#endif
+
 #include "xfconf-gvaluefuncs.h"
 #include "xfconf-common-private.h"
 #include "xfconf/xfconf.h"
@@ -163,7 +163,16 @@ xfconf_query_list_contents (GSList *sorted_contents, GHashTable *channel_content
         if (verbose)
         {
             property_value = g_hash_table_lookup (channel_contents, li->data);
-            string = _xfconf_string_from_gvalue (property_value);
+
+            if (XFCONF_TYPE_G_VALUE_ARRAY != G_VALUE_TYPE (property_value))
+            {
+                string = _xfconf_string_from_gvalue (property_value);
+            }
+            else
+            {
+                string = g_strdup ("<<UNSUPPORTED>>");
+            }
+
             g_print (format, (gchar *) li->data, string);
             g_free (string);
         }
@@ -222,6 +231,7 @@ static GOptionEntry entries[] =
         N_("Force array even if only one element"),
         NULL
     },
+/*
     {   "export", 'x', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &export_file,
         N_("Export channel to file"),
         NULL,
@@ -230,6 +240,7 @@ static GOptionEntry entries[] =
         N_("Import channel from file"),
         NULL,
     },
+*/
     {   "monitor", 'm', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &monitor,
         N_("Monitor a channel for property changes"),
         NULL,
@@ -247,6 +258,9 @@ main(int argc, char **argv)
     gint fd = -1;
     GOptionContext *context;
 
+#ifdef ENABLE_NLS
+    setlocale (LC_ALL, "");
+#endif
     xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 
     g_type_init();
@@ -263,13 +277,13 @@ main(int argc, char **argv)
 
     if(!g_option_context_parse(context, &argc, &argv, &cli_error))
     {
-        g_print("option parsing failed: %s\n", cli_error->message);
+        g_printerr("option parsing failed: %s\n", cli_error->message);
         return 1;
     }
 
     if(version)
     {
-        g_print("xfconf-query  %s\n", PACKAGE_VERSION);
+        g_print("xfconf-query %s\n", PACKAGE_VERSION);
         return 0;
     }
 
@@ -295,31 +309,31 @@ main(int argc, char **argv)
     /** Check if the property is specified */
     if(!property_name && !list && !export_file && !import_file && !monitor)
     {
-        g_print("No property specified, aborting...\n");
+        g_printerr("No property specified, aborting...\n");
         return 1;
     }
 
     if (create && reset)
     {
-        g_print("--create and --reset options can not be used together,\naborting...\n");
+        g_printerr("--create and --reset options can not be used together,\naborting...\n");
         return 1;
     }
 
     if ((create || reset) && (list))
     {
-        g_print("--create and --reset options can not be used together with\n --list\naborting...\n");
+        g_printerr("--create and --reset options can not be used together with\n --list\naborting...\n");
         return 1;
     }
 
     if (import_file && export_file)
     {
-        g_print("--import and --export options can not be used together,\naborting...\n");
+        g_printerr("--import and --export options can not be used together,\naborting...\n");
         return 1;
     }
 
     if ((import_file || export_file) && (list || property_name || create || reset))
     {
-        g_print("--import and --export options can not be used together with\n --create, --reset, --property and --list,\naborting...\n");
+        g_printerr("--import and --export options can not be used together with\n --create, --reset, --property and --list,\naborting...\n");
         return 1;
     }
 
@@ -354,8 +368,8 @@ main(int argc, char **argv)
 
             if(!xfconf_channel_get_property(channel, property_name, &value))
             {
-                g_print(_("Property \"%s\" does not exist on channel \"%s\".\n"),
-                        property_name, channel_name);
+                g_printerr(_("Property \"%s\" does not exist on channel \"%s\".\n"),
+                           property_name, channel_name);
                 return 1;
             }
             
